/* *******************  compute quantile of adaptive Neyman test********** */
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

/* global variables */
long L0 = 1, Nsim=100000, df =10000;
char sim='n';
   /* df is the degrees of freedom used to generate noise,
      when df = 10000, the data are generated from normal distribution */
void main(argc, argv)
int argc;
char **argv;
{
int c, n, nsim,  i, idum, j, Nmax;
double  *neyman, *x, result[15], tmp, sum, *teststat;
extern char *optarg;
char filename[15];
void sort(); 
FILE *out;
double *vector(), ran1(), gasdev(), aneyman();

 /* ********  Command OPTIONS  ******* */
  Nmax = 200;
  while ((c = getopt(argc, argv, "L:N:d:s:h")) !=EOF)
    switch(c){
       case 'L': sscanf(optarg, "%d",&L0);   break;
       case 'N': sscanf(optarg,"%d",&Nsim); break;
       case 'd': sscanf(optarg,"%d",&df); break;
       case 's': sscanf(optarg,"%1s",&sim); break;
       default: fprintf(stderr,"\n Legal option :\n");
		printf("-s n -N = 100000 -L 1 to compute P-values \n"); 
		printf("\t here L = number of P-values to be computed \n");
		printf("\t N = number of simulations to be used\n");
                printf("or -s y  -N = 100000 -d 10000 to");
		printf(" carry out simulations in the paper\n");
		printf("\t here d = degrees of freedom\n\n");
                exit(1);
      }

printf("%s", nsim);


/* *************  This block compute P-values of given vector *************** */
   if(sim < 'y') 
    { printf("\n \n Use a.out -h to see other options \n \n"); 
      printf("\t Compute P-values for a given vector based on %d simulations\n",
      		Nsim);
      printf("Enter n of the adaptive Neyman test statistc:   ");
      scanf("%d", &n);
      neyman =vector(0,Nsim);
      x      =vector(0,n);
      idum = -5319035;
      teststat = vector(0,L0);
      printf("Enter the vector of that P-values to be computed:  ");
      for(i=0; i < L0; i++)
	   scanf("%lf",teststat+i);
      printf("\n \n");
      fprintf(stderr, "Start computing (one dot per 1000 simulations)");
      for(nsim=0; nsim < Nsim; nsim++)       /* simulations */
        { for(i=0; i < n; i++) x[i] = gasdev(&idum);
	  neyman[nsim]  = aneyman(x,n);
	  if(nsim/1000*1000==nsim) fprintf(stderr, ".");
	}
      printf("\n\n");
      for(i=0; i < L0 ; i++)
	{ sum = 0.0;
	  for(j=0; j < Nsim; j++) 
	      if(neyman[j] > teststat[i]) sum += 1.0;
          tmp = sum/Nsim;           /*  P-values */
	  printf("\t P-values for teststat %10.4f       = %10.8f \n",
		teststat[i], tmp);
        }
       exit(1);
    }
/*  ************  End computing P-values   ******************* */


    neyman =vector(0,Nsim);
    x      =vector(0,Nmax);
    idum = -5319035;
    sprintf(filename, ".res%d", df);
    out       = fopen(filename, "w");
    fprintf(out,"Quantiles based on Nsim =%d, t-distribution with df = %d \n"
    ,  Nsim, df);
    fprintf(out, "When df > 100, it is indeed generated from");
    fprintf(out, "the normal distribution\n \n \n");
    fprintf(out, "  n   &  0.00001 & 0.00005 &  0.0001 & 0.00025 & 0.0005 &");
    fprintf(out, " 0.001 & 0.0025 & 0.005 & 0.01 & ");
    fprintf(out, " 0.025 & 0.05 & 0.10 & 0.25 & 0.50 \\\\ \n");


  /* ******************SIMULATION STARTS ********************* */
 for(n=5; n < Nmax+1; n = n+5)
  {
    for(nsim=0; nsim < Nsim; nsim++)
       { 
         if(df > 100)  /* Gaussian error */
	     for(i=0; i < n; i++) x[i] = gasdev(&idum);
         else         /* t-distribution with df */
	     { for (i = 0; i < n ; i++)
		{ sum =0.0;
		  for(j=0; j < df/2; j++) sum += log(ran1(&idum));
		  sum *= -2.0;                 /*sum of chi_2^2 */
                  if(df/2*2 < df) {tmp = gasdev(&idum); sum += tmp*tmp;}
	          x[i] = gasdev(&idum)/sqrt(sum/df); 
                }

             }
	 neyman[nsim]  = aneyman(x,n);
       }    /*end loop with nsim */    
     sort(neyman, Nsim);

     j= Nsim*(1-0.00001), result[0] = neyman[j]; 
     j= Nsim*(1-0.00005), result[1] = neyman[j]; 
     j= Nsim*(1-0.00010), result[2] = neyman[j]; 
     j= Nsim*(1-0.00025), result[3] = neyman[j]; 
     j= Nsim*(1-0.00050), result[4] = neyman[j]; 
     j= Nsim*(1-0.00100), result[5] = neyman[j]; 
     j= Nsim*(1-0.00250), result[6] = neyman[j]; 
     j= Nsim*(1-0.00500), result[7] = neyman[j]; 
     j= Nsim*(1-0.01000), result[8] = neyman[j]; 
     j= Nsim*(1-0.02500), result[9] = neyman[j]; 
     j= Nsim*(1-0.05000), result[10] = neyman[j]; 
     j= Nsim*(1-0.10000), result[11] = neyman[j]; 
     j= Nsim*(1-0.25000), result[12] = neyman[j]; 
     j= Nsim*(1-0.50000), result[13] = neyman[j]; 
	
     fprintf(out, "%d & ", n);
     tmp = df/(df-2.0)*sqrt((df-1.0)/(df-4.0));
     sum = log( log ( ( double) n ) );
     sum = (2*sum + 0.5*log(sum)-1.26);
     sum *= (1-1.0/tmp);
     /* printf("df = %d  tmp = %10.6f sum = %10.6f\n", df, tmp, sum); */
				/*adjusting for t-distribution */
    
     for(i=0; i < 8; i++) 
	   fprintf(out, "%6.2f&", result[i]/tmp-sum);
     fprintf(out, "\n     ");
     for(i=8; i < 14; i++) 
	   fprintf(out, "%6.2f&", result[i]/tmp-sum);
     fprintf(out, "\\\\ \n");
	    
  }  /*end the loop with n*/

}
/* ************************ADPATIVE NEYMAN.C*********************************
Compute Adaptive Neyman test statisti
x: n by 1 vector, is double.  
************************************************************************** */
double aneyman(x,n)
double *x; 
int n;
{
  int i, mopt;
  double sum, teststat, valueopt, valuenew;
  double logn, constant;

  constant = sqrt(2/3.14159);          /* sqrt(2/pi) */

	/*  Computing Adaptive Neyman statistic  */
  mopt = 1; sum = x[0]*x[0]; valueopt = sum - 1.0;
  for(i=1; i < n; i++)
	  { sum += x[i]*x[i];
            valuenew = (sum-(i+1))/sqrt((double) (i+1));
	    if(valuenew > valueopt)
	           {mopt = i+1; valueopt = valuenew;}
          }
         teststat = valueopt/sqrt(2.0);
	 logn = log( log ( ( double) n ) );
	 teststat = sqrt(2*logn)*teststat - (2*logn + 0.5*log(logn)-1.26);

  return teststat;
}

/* ********************** VECTOR.C **************** */
double *vector(nl, nh)
int nl, nh;
{  double *v;
   v = (double *)malloc((unsigned) (nh-nl+1) * sizeof(double));
      return v-nl;
}


/* ******************** UNIFORM RANDOM NUMBER GENERATOR: RAN1.c *******
*/
#include <stdlib.h>
#define M1 259200
#define IA1 7141
#define IC1 54773
#define RM1 (1.0/M1)
#define M2 134456
#define IA2 8121
#define IC2 28411
#define RM2 (1.0/M2)
#define M3 243000
#define IA3 4561
#define IC3 51349

double ran1(idum)
int *idum;

                    /* returns a uniform random deviate between 0.0 and
1.0.
                       Set idum to any negative value to initialize or
                       reinitialize the sequence  */

{
    static long ix1,ix2,ix3;
    static double r[98];
    double temp;
    static int iff=0;
    int j;
    void nrerror();

    if (*idum < 0 || iff == 0) {
         iff=1;
         ix1=(IC1-(*idum)) % M1;
         ix1=(IA1*ix1+IC1) % M1;
         ix2=ix1 % M2;
         ix1=(IA1*ix1+IC1) % M1;
         ix3=ix1 % M3;
         for (j=1;j<=97;j++) {
               ix1=(IA1*ix1+IC1) % M1;
               ix2=(IA2*ix2+IC2) % M2;
               r[j]=(ix1+ix2*RM2)*RM1;
           }
           *idum=1;
    }
   ix1=(IA1*ix1+IC1) % M1;
    ix2=(IA2*ix2+IC2) % M2;
    ix3=(IA3*ix3+IC3) % M3;
    j=1 + ((97*ix3)/M3);
    if (j > 97 || j < 1) nrerror("RAN1: This cannot happen.");
    temp=r[j];
    r[j]=(ix1+ix2*RM2)*RM1;
    return temp;
}

/* ***************** NORMAL RANDOM GENERATOR: GASDEV.C *********** */
double gasdev(idum)
int *idum;
                   /* returns a normally distributed deviate with mean 0
                     and variance 1, using ran1(idum) as the source of uniform
                      deviates    */

{
    static int iset=0;
    static double gset;
    double fac,r,v1,v2;
    double ran1();

    if (iset == 0)  {
        do {
            v1=2.0*ran1(idum)-1.0;
            v2=2.0*ran1(idum)-1.0;
            r=v1*v1+v2*v2;
        } while (r >= 1.0 || r == 0.0);
        fac= sqrt(-2.0*log(r)/r);
        gset=v1*fac;
        iset=1;
        return v2*fac;
    }  else {
         iset=0;
         return gset;
    }
}

void sort(ra,n)
int n;
double *ra;
/*index of ra is 0:n-1*/
{
        int l,j,ir,i;
        double rra;
 
        l=(n >> 1)+1;
        ir=n;
        for (;;) {
                if (l > 1)
                        rra=ra[--l-1];
                else {
                        rra=ra[ir-1];
                        ra[ir-1]=ra[0];
                        if (--ir == 1) {
                                ra[0]=rra;
                                return;
                        }
                }
                i=l;
                j=l << 1;
                while (j <= ir) {
                        if (j < ir && ra[j-1] < ra[j]) ++j;
                        if (rra < ra[j-1]) {
                                ra[i-1]=ra[j-1];
                                j += (i=j);
                        }
                        else j=ir+1;
                }
                ra[i-1]=rra;
        }
}

/*    *********************NRERROR.C ******************** */
void nrerror(error_text)
     /* prints an error message */

char error_text[];
{
   fprintf(stderr,"Numerical recipes run-time error...\n");
   fprintf(stderr,"%s\n",error_text);
   fprintf(stderr,"...now exiting to system...\n");
   exit(1);
}


